from .joystick import Joystick
from time import sleep

class JoystickTest:
    def __init__():
        """
        This is the test class for the Joystick class.
        To use this, run JoystickTest.test(xChannel, yChannel, zPin).
        """
        pass

    @classmethod
    def test(self, xchn, ychn, zpin):
        """
        This will print the value of x, y, and z in a loop.
        """
        joystick = Joystick(xchn, ychn, zpin)

        try:
            while True:
                x, y, z = joystick.read()
                print('x: {}, y: {}, z: {}'.format(x,y,z))
                sleep(0.1)
        except KeyboardInterrupt:
            joystick.close()



import RPi.GPIO as GPIO
from .light_errors import RGBLedError

class RGBLed:
    """
    This class is used to control a RGB LED via the GPIO.
    Please make sure you have 220 Ohm resistors between the 3 gpio pins
    and the LED.
    """

    def __init__(self, red_pin, green_pin, blue_pin):
        """
        This initiates the pins needed to work with the RGB LED.
        """
        try:
            self.red_pin = int(red_pin)
            self.green_pin = int(green_pin)
            self.blue_pin = int(blue_pin)

            GPIO.setmode(GPIO.BOARD)
            GPIO.setup(self.red_pin, GPIO.OUT)
            GPIO.setup(self.green_pin, GPIO.OUT)
            GPIO.setup(self.blue_pin, GPIO.OUT)

            GPIO.output(self.red_pin, GPIO.OUT)
            GPIO.output(self.green_pin, GPIO.OUT)
            GPIO.output(self.blue_pin, GPIO.OUT)

            self.pwm_red = GPIO.PWM(self.red_pin, 2000)
            self.pwm_green = GPIO.PWM(self.green_pin, 2000)
            self.pwm_blue = GPIO.PWM(self.blue_pin, 2000)
        except:
            raise RGBLedError("Error during the initiation of the RGB_LED class.")

    def on(self, red_val=0, green_val=0, blue_val=0):
        """
        This will turn on the RGB LED. To change the default color, change red_val,
        green_val, and blue_val when calling the function.
        """
        try:
            self.pwm_red.start(red_val)
            self.pwm_green.start(green_val)
            self.pwm_blue.start(blue_val)
        except:
            raise RGBLedError("Error while turning the RGB LED on.")

    def set_color(self, red_val, green_val, blue_val):
        """
        This will set the color of the RGB LED.
        Keep in mind that this uses the RGB color scale.
        """
        try:
            self.pwm_red.ChangeDutyCycle(red_val)
            self.pwm_green.ChangeDutyCycle(green_val)
            self.pwm_blue.ChangeDutyCycle(blue_val)
        except:
            raise RGBLedError("Error while changing the color of the RGB LED.")

    def off(self):
        """
        This will turn the RGB LED off.
        """
        try:
            self.pwm_red.stop()
            self.pwm_green.stop()
            self.pwm_blue.stop()
        except:
            raise RGBLedError("Error while turning the RGB LED off.")



# Imports the LedError and RGBLedError class.
from .light.light_errors import LedError, RGBLedError
# Imports the Led class.
from .light.light import Led
# Imports the LedTest class, the test class for the LED
from .light.test_light import LedTest
# Imports the RGBLed class.
from .light.rgb_light import RGBLed
# Imports the RGBLedTest class, the test class for the RGB LED.
from .light.test_rgb import RGBLedTest

# Imports the ButtonError class.
from .button.button_errors import ButtonError
# Imports the Button class.
from .button.button import Button
# Imports the ButtonTest calss, the test class for the button.
from .button.test_button import ButtonTest

# Imports the BuzzerError class.
from .buzzer.buzzer_errors import BuzzerError
# Imports the Buzzer class.
from .buzzer.buzzer import Buzzer
# Imports the BuzzerTest class, the test class for the buzzer.
from .buzzer.test_buzzer import BuzzerTest

# Imports the ADCError class.
from .ADC.ADC_errors import ADCError
# Imports the PCF class.
from .ADC.ADC import PCF
# Imports ADCTest, the test class for the PCF.
from .ADC.test_ADC import ADCTest

# Imports the PotentiometerError class.
from .variable_resistor.resistor_errors import ResistorError
# Imports the Potentiometer class.
from .variable_resistor.resistor import VariableResistor
# Imports PotentiometerTest, the test class for the potentiometer.
from .variable_resistor.test_resistor import ResistorTest

# Imports the JoystickError class.
from .joystick.joystick_errors import JoystickError
# Imports the Joystick class.
from .joystick.joystick import Joystick
# Imports JoystickTest, the test class for the joystick.
from .joystick.test_joystick import JoystickTest



import RPi.GPIO as GPIO
from .buzzer_errors import BuzzerError

class Buzzer:
    """
    This class is made to control an active buzzer via the GPIO on the given pin.
    See the documentation on how to wire an active buzzer to work with this class.
    """
    def __init__(self, pin):
        """
        This initiates the buzzer on the given pin.
        """
        self.pin = int(pin)
        GPIO.setmode(GPIO.BOARD)
        GPIO.setup(self.pin, GPIO.OUT)

    def on(self):
        """
        This turns the buzzer on.
        """
        GPIO.output(self.pin, GPIO.HIGH)

    def off(self):
        """
        This turns the buzzer off.
        """
        GPIO.output(self.pin, GPIO.LOW)



class LedError(Exception):
    """
    Raised when there is an error while working with the LED class.
    """
    def __init__(self):
        pass

class RGBLedError(Exception):
    """
    Raised when there is an error while working with the RGB_LED class.
    """
    def __init__(self):
        pass



from math import log
from ..ADC.ADC import PCF
from ..ADC.ADC_errors import ADCError
from .resistor_errors import ResistorError

class VariableResistor:
    """
    This class can be used to work with potentiometers, photoresistors,
    and thermistors.
    """
    def __init__(self, chn):
        """
        Initiates the VariableResistor class on the given channel.
        """
        self.channel = chn
        self.adc = PCF()
        try:
            self.adc.is_connected()
        except ADCError:
            raise ResistorError("Failed to connect to your ADC.")

    def read(self):
        """
        Reads the value on the given channel in the initiation.
        """
        value = self.adc.read(self.channel)
        voltage = value / 255.0 * 3.3
        resistance = 10*voltage/(3.3-voltage)
        return value, voltage, resistance

    def temp(self, resistance):
        """
        This is used for the thermistor, to turn the resistance value into temperature
        in farhenheit.
        """
        temp = (1/(1/(273.15 + 25) + math.log(resistance/10)/3950.0)-273.15)*(9/5)+32 # Farhenheit
        return temp

    def close(self):
        """
        Used to close the connection to the ADC.
        """
        self.adc.close()



import RPi.GPIO as GPIO
from .light_errors import LedError

class Led:
    """
    This is a class used to control LED's directly connected to the GPIO via a pin given.
    See the documentation for an example of how to wire the LED.
    """
    def __init__(self, pin):
        """
        This initates the LED on the given pin, setting it into the output mode,
        making sure it is off, and setting the PWM up so that the LED can be dimmed.
        """
        try:
            self.pin = int(pin)

            GPIO.setmode(GPIO.BOARD)
            GPIO.setup(self.pin, GPIO.OUT)
            GPIO.output(self.pin, GPIO.LOW)

            self.led_dim = GPIO.PWM(self.pin, 500)
        except:
            raise LedError("Error during the initiation of the LED class.")

    def on(self, brightness=100):
        """
        Turns the defined LED on, the brightness is set by default to 100%.
        """
        try:
            self.led_dim.start(brightness)
        except:
            raise LedError("Error while turning the LED on.")

    def off(self):
        """
        Turns the defined LED off.
        """
        try:
            self.led_dim.stop()
        except:
            raise LedError("Error while turning the LED off.")

    def dim(self, brightness):
        """
        Dims the definied LED. Keep in mind, that if you don't first turn the
        LED on this will error out.
        """
        if brightness < 0:
            brightness = 0
        elif brightness > 100:
            brightness = 100
        else:
            pass

        try:
            self.led_dim.ChangeDutyCycle(brightness)
        except:
            raise LedError("Error while dimming the LED. Make sure you have turned the LED on.")



from .button import Button
from time import sleep

class ButtonTest:
    def __init__():
        """
        This is a test class for Button. To call this run ButtonTest.test(pin).
        Initating this does nothing.
        """
        pass

    @classmethod
    def test(self, pin):
        """
        This will run through a loop, checking if the defined button was pressed
        every second. It will print True if it is, otherwise it will pring False.
        """
        button = Button(pin)

        try:
            while True:
                pressed = button.is_pressed()
                print(pressed)
                sleep(1)
        except KeyboardInterrupt:
            pass



import RPi.GPIO as GPIO
from ..ADC.ADC import PCF
from ..ADC.ADC_errors import ADCError

class Joystick:
    """
    This class is used to get the values from a joystick.
    """
    def __init__(self, xchn, ychn, zpin):
        """
        Initiates the Joystick class on the given two channels and one pin.
        """
        self.xChannel = xchn
        self.yChannel = ychn
        self.zPin = zpin

        self.adc = PCF()

        try:
            self.adc.is_connected()
        except ADCError:
            raise JoystickError("Failed to connect to your ADC.")

        GPIO.setmode(GPIO.BOARD)
        GPIO.setup(self.zPin, GPIO.IN, GPIO.PUD_UP)

    def read(self):
        """
        Reads the values of the x, y, and z axis of the joystick.
        """
        x = self.adc.read(self.xChannel)
        y = self.adc.read(self.yChannel)
        z = GPIO.input(self.zPin)
        return x, y, z

    def close(self):
        """
        closes the connection with the adc.
        """
        self.adc.close()



from .rgb_light import RGBLed
from time import sleep
from random import randint

class RGBLedTest:
    def __init__():
        """
        This is a test function to see if you have properly setup
        your RGB LED. It will randomly change colors every second
        until you stop it.
        """
        pass

    @classmethod
    def test(self, rpin, gpin, bpin):
        """
        This will turn the RGB LED on, and then randomly select a color to set the
        RGB LED to.
        """
        try:
            led = RGBLed(rpin, gpin, bpin)
            led.on()

            while True:
                red = randint(0, 100)
                green = randint(0, 100)
                blue = randint(0, 100)
                led.set_color(red, green, blue)
                sleep(1)
        except KeyboardInterrupt:
            print(" No errors were raised while this program ran.")
        else:
            pass
        finally:
            led.off()



from .buzzer import Buzzer
from time import sleep

class BuzzerTest:
    def __init__():
        """
        This is a test class for Buzzer. To call this run BuzzerTest.test(pin).
        Initating this does nothing.
        """
        pass

    @classmethod
    def test(self, pin):
        """
        This will turn the buzzer on for one second. If this works you will hear a beep.
        """
        buzzer = Buzzer(pin)

        try:
            buzzer.on()
            sleep(1)
            buzzer.off()
        except KeyboardInterrupt:
            buzzer.off()
        finally:
            buzzer.off()



from time import sleep
from .light import Led

class LedTest:
    def __init__():
        """
        This is a test class. To call this run LedTest.test(pin).
        Do not initate this, as it will do nothing.
        """
        pass

    @classmethod
    def test(cls, pin):
        """
        This will loop turning the LED on and off, and dimming it.
        """
        try:
            led = Led(pin)

            while True:
                led.on(0)
                for x in range(0,100,1):
                    led.dim(x)
                    sleep(.1)
                for x in range(100,0,-1):
                    led.dim(x)
                    sleep(.1)
                led.off()
                sleep(1)
        except KeyboardInterrupt:
            print(" No errors were encountered while this test of your LED was ran.")
        else:
            pass
        finally:
            led.off()



import smbus
from .ADC_errors import ADCError

class PCF():
    """
    This class is used to work with the PCF8591 module, an ADC.
    """
    def __init__(self):
        """
        This initiates the ADC.
        """
        self.cmd = 0x40
        self.address = 0x48
        try:
            self.bus = smbus.SMBus(1)
        except:
            raise ADCError("Error while initiating the PCF class.")

    def is_connected(self):
        """
        This tests to see if the connection was successful or not.
        """
        try:
            self.bus.write_byte(self.address, 0)
        except:
            raise ADCError("No ADC found.")

    def close(self):
        """
        This closes the ADC.
        """
        self.bus.close()

    def read(self, chn):
        """
        This reads the value from the specified channel on the ADC.
        """
        value = self.bus.read_byte_data(self.address, self.cmd+chn)
        return value

    def write(self, chn, value):
        """
        This writes a value to the specified channel on the ADC.
        """
        self.bus.write_byte_data(self.address, self.cmd+chn, value)



from .resistor import VariableResistor
from time import sleep

class ResistorTest:
    def __init__():
        """
        This is the test class for the VariableResistor class.
        To use this, run ResistorTest.test(channel).
        """
        pass

    @classmethod
    def test(self, chn):
        """
        This will return the value, voltage, and resistance in a loop.
        """
        resistor = VariableResistor(chn)

        try:
            while True:
                value, voltage, resistance = resistor.read()
                print('Value: {}, Voltage: {}, Resistance: {}'.format(value, round(voltage, 1), round(resistance, 1)))
                sleep(0.1)
        except KeyboardInterrupt:
            resistor.close()



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="raspi_components",
    version="1.3.2",
    author="builderdev212",
    description="A python package for using certain components with the raspberry pi.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    keywords="raspberry pi",
    url="https://github.com/builderdev212/package_tutorial",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3',
)

# Development Status :: 1 - Planning
# Development Status :: 2 - Pre-Alpha
# Development Status :: 3 - Alpha
# Development Status :: 4 - Beta
# Development Status :: 5 - Production/Stable
# Development Status :: 6 - Mature
# Development Status :: 7 - Inactive



from .ADC import PCF
from time import sleep

class ADCTest:
    def __init__():
        """
        This is the test class for the ADC. To use this,
        run ADCTest.test().
        """
        pass

    @classmethod
    def test(self):
        """
        This tests whether the adc can be connected too. It will
        print either success or failure.
        """
        adc = PCF()
        try:
            adc.is_connected()
        except:
            print("Failure to connect to the ADC.")
        else:
            print("Success.")



import RPi.GPIO as GPIO
from .button_errors import ButtonError

class Button:
    def __init__(self, pin):
        """
        Initiates the button class on a certain gpio pin.
        """
        try:
            self.pin = int(pin)
            GPIO.setmode(GPIO.BOARD)
            GPIO.setup(self.pin, GPIO.IN, pull_up_down=GPIO.PUD_UP)
        except:
            raise ButtonError("Error while initating the Button class.")

    def is_pressed(self):
        """
        Checks to see if the defined button is clicked.
        """
        try:
            if GPIO.input(self.pin) == GPIO.LOW:
                return True
            else:
                return False
        except:
            raise ButtonError("Error while checking to see if your definied button was pressed.")
